#include "Platform/Platform.hpp"
#include <Agents/Agent.hpp>
#include <Agents/Perception/Perception.hpp>
#include <Environment/PictureEnvironment.hpp>

const sf::Vector2f DEFAUMT_SIMZE = sf::Vector2f(800, 600);
int PERCEPTION_SIZE = 4;

PictureEnvironment::PictureEnvironment()
{
}

PictureEnvironment::PictureEnvironment(std::string& iFilePath, sf::RenderWindow* iWindow, util::Platform* iPlatform) :
	fSize(iWindow->getSize())
{

	fFilePath = iFilePath;
	//sf::Texture fLoadedImage;
	if (!fTextureImage.loadFromFile(fFilePath))
	{
		std::cout << "\n COULDNT LOAD PICTURE\n";
	}
	fImage = fTextureImage.copyToImage();

	//memccpy(&fFilePath, &iFilePath, *iFilePath., 10);
	std::cout << "Environement pour " << fFilePath;
	fPlatform = iPlatform;
	fWindow = iWindow;
	float screenScalingFactor = fPlatform->getScreenScalingFactor(fWindow->getSystemHandle());
	fWindow->create(sf::VideoMode(800.0f * screenScalingFactor, 600.0f * screenScalingFactor), "SFML works!");
	fPlatform->setIcon(fWindow->getSystemHandle());
	fInitialSize.x = 800.0f * screenScalingFactor;
	fInitialSize.y = 600.0f * screenScalingFactor;
	//fShape = sf::RectangleShape(fSize);
	fShape = sf::RectangleShape(DEFAUMT_SIMZE);

	fShape.setFillColor(sf::Color::White);

	fShape.setTexture(&fTextureImage);
}

Perception PictureEnvironment::getPerception(int iX, int iY)
{
	Perception vPerception = Perception(PERCEPTION_SIZE, PERCEPTION_SIZE);
	iX++;
	iY++;
	return vPerception;
}

void PictureEnvironment::makeAgents(int iNumber)
{
	Agent vTmpAgent;
	fAgents = std::list<Agent>(iNumber);
	fAgentsIterator = fAgents.begin();

	for (int i = 0; i < iNumber; i++)
	{
		vTmpAgent = Agent(this);
		fAgents.insert(fAgentsIterator, vTmpAgent);
	}
}

bool PictureEnvironment::loop()
{
	if (fWindow->isOpen())
	{
		while (fWindow->pollEvent(fEvent))
		{
			if (fEvent.type == sf::Event::Closed)
				fWindow->close();
		}

		fAgentsIterator = fAgents.begin();
		for (uint i = 0; i < fAgents.size(); i++)
		{
			fAgentsIterator->loop();
		}

		fTextureImage.loadFromImage(fImage);
		fShape.setTexture(&fTextureImage);
		fWindow->clear();
		fWindow->draw(fShape);
		fWindow->display();
	}
	else
	{
		std::cout << "\nBah on ferme";
		return false;
	}
	return true;
}

void PictureEnvironment::displayLockInput()
{
	fWindow->create(sf::VideoMode(fInitialSize.x, fInitialSize.y), "Last peek");
	fWindow->clear();
	fWindow->draw(fShape);
	fWindow->display();
	int i;
	std::cin >> i;
}

void PictureEnvironment::paint(int iX, int iY, const sf::Color iColour)
{
	//std::cout << "juste avant : " << iX << " et " << iY;
	fImage.setPixel(iX, iY, iColour);
}